@@ -11,6 +11,10 @@ if is_local and os.name == 'nt':
         os.add_dll_directory(os.path.dirname(__file__))
 
 
+class GuiImportError(ImportError):
+    pass
+
+
 from .version import ELECTRUM_VERSION
 from .util import format_satoshis
 from .wallet import Wallet
@@ -25,6 +29,11 @@ from . import daemon
 from .transaction import Transaction
 from .plugin import BasePlugin
 from .commands import Commands, known_commands
+from .logging import get_logger
 
 
 __version__ = ELECTRUM_VERSION
+
+_logger = get_logger(__name__)
+if not __debug__:
+    _logger.warning(f"__debug__ is False. running with asserts disabled!")
