@@ -24,38 +24,58 @@
 # SOFTWARE.
 
 import base64
+import binascii
 import os
 import sys
 import hashlib
 import hmac
-from typing import Union
+from typing import Union, Mapping, Optional
 
-import pyaes
-
-from .util import assert_bytes, InvalidPassword, to_bytes, to_string, WalletFileException
+from .util import assert_bytes, InvalidPassword, to_bytes, to_string, WalletFileException, versiontuple
 from .i18n import _
+from .logging import get_logger
 
 
+_logger = get_logger(__name__)
+
+
+HAS_PYAES = False
+try:
+    import pyaes
+except Exception:
+    pass
+else:
+    HAS_PYAES = True
+
 HAS_CRYPTODOME = False
+MIN_CRYPTODOME_VERSION = "3.7"
 try:
+    import Cryptodome
+    if versiontuple(Cryptodome.__version__) < versiontuple(MIN_CRYPTODOME_VERSION):
+        _logger.warning(f"found module 'Cryptodome' but it is too old: {Cryptodome.__version__}<{MIN_CRYPTODOME_VERSION}")
+        raise Exception()
     from Cryptodome.Cipher import ChaCha20_Poly1305 as CD_ChaCha20_Poly1305
     from Cryptodome.Cipher import ChaCha20 as CD_ChaCha20
     from Cryptodome.Cipher import AES as CD_AES
-except:
+except Exception:
     pass
 else:
     HAS_CRYPTODOME = True
 
 HAS_CRYPTOGRAPHY = False
+MIN_CRYPTOGRAPHY_VERSION = "2.1"
 try:
     import cryptography
+    if versiontuple(cryptography.__version__) < versiontuple(MIN_CRYPTOGRAPHY_VERSION):
+        _logger.warning(f"found module 'cryptography' but it is too old: {cryptography.__version__}<{MIN_CRYPTOGRAPHY_VERSION}")
+        raise Exception()
     from cryptography import exceptions
     from cryptography.hazmat.primitives.ciphers import Cipher as CG_Cipher
     from cryptography.hazmat.primitives.ciphers import algorithms as CG_algorithms
     from cryptography.hazmat.primitives.ciphers import modes as CG_modes
     from cryptography.hazmat.backends import default_backend as CG_default_backend
     import cryptography.hazmat.primitives.ciphers.aead as CG_aead
-except:
+except Exception:
     pass
 else:
     HAS_CRYPTOGRAPHY = True
@@ -65,10 +85,35 @@ if not (HAS_CRYPTODOME or HAS_CRYPTOGRAPHY):
     sys.exit(f"Error: at least one of ('pycryptodomex', 'cryptography') needs to be installed.")
 
 
+def version_info() -> Mapping[str, Optional[str]]:
+    ret = {}
+    if HAS_PYAES:
+        ret["pyaes.version"] = ".".join(map(str, pyaes.VERSION[:3]))
+    else:
+        ret["pyaes.version"] = None
+    if HAS_CRYPTODOME:
+        ret["cryptodome.version"] = Cryptodome.__version__
+        if hasattr(Cryptodome, "__path__"):
+            ret["cryptodome.path"] = ", ".join(Cryptodome.__path__ or [])
+    else:
+        ret["cryptodome.version"] = None
+    if HAS_CRYPTOGRAPHY:
+        ret["cryptography.version"] = cryptography.__version__
+        if hasattr(cryptography, "__path__"):
+            ret["cryptography.path"] = ", ".join(cryptography.__path__ or [])
+    else:
+        ret["cryptography.version"] = None
+    return ret
+
+
 class InvalidPadding(Exception):
     pass
 
 
+class CiphertextFormatError(Exception):
+    pass
+
+
 def append_PKCS7_padding(data: bytes) -> bytes:
     assert_bytes(data)
     padlen = 16 - (len(data) % 16)
@@ -97,10 +142,12 @@ def aes_encrypt_with_iv(key: bytes, iv: bytes, data: bytes) -> bytes:
         cipher = CG_Cipher(CG_algorithms.AES(key), CG_modes.CBC(iv), backend=CG_default_backend())
         encryptor = cipher.encryptor()
         e = encryptor.update(data) + encryptor.finalize()
-    else:
+    elif HAS_PYAES:
         aes_cbc = pyaes.AESModeOfOperationCBC(key, iv=iv)
         aes = pyaes.Encrypter(aes_cbc, padding=pyaes.PADDING_NONE)
         e = aes.feed(data) + aes.feed()  # empty aes.feed() flushes buffer
+    else:
+        raise Exception("no AES backend found")
     return e
 
 
@@ -113,22 +160,18 @@ def aes_decrypt_with_iv(key: bytes, iv: bytes, data: bytes) -> bytes:
         cipher = CG_Cipher(CG_algorithms.AES(key), CG_modes.CBC(iv), backend=CG_default_backend())
         decryptor = cipher.decryptor()
         data = decryptor.update(data) + decryptor.finalize()
-    else:
+    elif HAS_PYAES:
         aes_cbc = pyaes.AESModeOfOperationCBC(key, iv=iv)
         aes = pyaes.Decrypter(aes_cbc, padding=pyaes.PADDING_NONE)
         data = aes.feed(data) + aes.feed()  # empty aes.feed() flushes buffer
+    else:
+        raise Exception("no AES backend found")
     try:
         return strip_PKCS7_padding(data)
     except InvalidPadding:
         raise InvalidPassword()
 
 
-def EncodeAES_base64(secret: bytes, msg: bytes) -> bytes:
-    """Returns base64 encoded ciphertext."""
-    e = EncodeAES_bytes(secret, msg)
-    return base64.b64encode(e)
-
-
 def EncodeAES_bytes(secret: bytes, msg: bytes) -> bytes:
     assert_bytes(msg)
     iv = bytes(os.urandom(16))
@@ -136,11 +179,6 @@ def EncodeAES_bytes(secret: bytes, msg: bytes) -> bytes:
     return iv + ct
 
 
-def DecodeAES_base64(secret: bytes, ciphertext_b64: Union[bytes, str]) -> bytes:
-    ciphertext = bytes(base64.b64decode(ciphertext_b64))
-    return DecodeAES_bytes(secret, ciphertext)
-
-
 def DecodeAES_bytes(secret: bytes, ciphertext: bytes) -> bytes:
     assert_bytes(ciphertext)
     iv, e = ciphertext[:16], ciphertext[16:]
@@ -149,8 +187,8 @@ def DecodeAES_bytes(secret: bytes, ciphertext: bytes) -> bytes:
 
 
 PW_HASH_VERSION_LATEST = 1
-KNOWN_PW_HASH_VERSIONS = (1, 2, )
-SUPPORTED_PW_HASH_VERSIONS = (1, )
+KNOWN_PW_HASH_VERSIONS = (1, 2,)
+SUPPORTED_PW_HASH_VERSIONS = (1,)
 assert PW_HASH_VERSION_LATEST in KNOWN_PW_HASH_VERSIONS
 assert PW_HASH_VERSION_LATEST in SUPPORTED_PW_HASH_VERSIONS
 
@@ -223,7 +261,10 @@ def pw_decode_bytes(data: str, password: Union[bytes, str], *, version:int) -> b
     """base64 ciphertext -> plaintext bytes"""
     if version not in KNOWN_PW_HASH_VERSIONS:
         raise UnexpectedPasswordHashVersion(version)
-    data_bytes = bytes(base64.b64decode(data))
+    try:
+        data_bytes = bytes(base64.b64decode(data, validate=True))
+    except binascii.Error as e:
+        raise CiphertextFormatError("ciphertext not valid base64") from e
     return _pw_decode_raw(data_bytes, password, version=version)
 
 
@@ -240,7 +281,10 @@ def pw_encode_with_version_and_mac(data: bytes, password: Union[bytes, str]) ->
 
 def pw_decode_with_version_and_mac(data: str, password: Union[bytes, str]) -> bytes:
     """base64 ciphertext -> plaintext bytes"""
-    data_bytes = bytes(base64.b64decode(data))
+    try:
+        data_bytes = bytes(base64.b64decode(data, validate=True))
+    except binascii.Error as e:
+        raise CiphertextFormatError("ciphertext not valid base64") from e
     version = int(data_bytes[0])
     encrypted = data_bytes[1:-4]
     mac = data_bytes[-4:]
@@ -292,6 +336,10 @@ def ripemd(x):
         md.update(x)
         return md.digest()
     except BaseException:
+        # ripemd160 is not guaranteed to be available in hashlib on all platforms.
+        # Historically, our Android builds had hashlib/openssl which did not have it.
+        # see https://github.com/spesmilo/electrum/issues/7093
+        # We bundle a pure python implementation as fallback that gets used now:
         from . import ripemd
         md = ripemd.new(x)
         return md.digest()
@@ -315,6 +363,8 @@ def chacha20_poly1305_encrypt(
     assert isinstance(nonce, (bytes, bytearray))
     assert isinstance(associated_data, (bytes, bytearray, type(None)))
     assert isinstance(data, (bytes, bytearray))
+    assert len(key) == 32, f"unexpected key size: {len(key)} (expected: 32)"
+    assert len(nonce) == 12, f"unexpected nonce size: {len(nonce)} (expected: 12)"
     if HAS_CRYPTODOME:
         cipher = CD_ChaCha20_Poly1305.new(key=key, nonce=nonce)
         if associated_data is not None:
@@ -338,6 +388,8 @@ def chacha20_poly1305_decrypt(
     assert isinstance(nonce, (bytes, bytearray))
     assert isinstance(associated_data, (bytes, bytearray, type(None)))
     assert isinstance(data, (bytes, bytearray))
+    assert len(key) == 32, f"unexpected key size: {len(key)} (expected: 32)"
+    assert len(nonce) == 12, f"unexpected nonce size: {len(nonce)} (expected: 12)"
     if HAS_CRYPTODOME:
         cipher = CD_ChaCha20_Poly1305.new(key=key, nonce=nonce)
         if associated_data is not None:
@@ -357,14 +409,33 @@ def chacha20_encrypt(*, key: bytes, nonce: bytes, data: bytes) -> bytes:
     assert isinstance(key, (bytes, bytearray))
     assert isinstance(nonce, (bytes, bytearray))
     assert isinstance(data, (bytes, bytearray))
-    assert len(nonce) == 8, f"unexpected nonce size: {len(nonce)} (expected: 8)"
+    assert len(key) == 32, f"unexpected key size: {len(key)} (expected: 32)"
+    assert len(nonce) in (8, 12), f"unexpected nonce size: {len(nonce)} (expected: 8 or 12)"
     if HAS_CRYPTODOME:
         cipher = CD_ChaCha20.new(key=key, nonce=nonce)
         return cipher.encrypt(data)
     if HAS_CRYPTOGRAPHY:
-        nonce = bytes(8) + nonce  # cryptography wants 16 byte nonces
+        nonce = bytes(16 - len(nonce)) + nonce  # cryptography wants 16 byte nonces
         algo = CG_algorithms.ChaCha20(key=key, nonce=nonce)
         cipher = CG_Cipher(algo, mode=None, backend=CG_default_backend())
         encryptor = cipher.encryptor()
         return encryptor.update(data)
     raise Exception("no chacha20 backend found")
+
+
+def chacha20_decrypt(*, key: bytes, nonce: bytes, data: bytes) -> bytes:
+    assert isinstance(key, (bytes, bytearray))
+    assert isinstance(nonce, (bytes, bytearray))
+    assert isinstance(data, (bytes, bytearray))
+    assert len(key) == 32, f"unexpected key size: {len(key)} (expected: 32)"
+    assert len(nonce) in (8, 12), f"unexpected nonce size: {len(nonce)} (expected: 8 or 12)"
+    if HAS_CRYPTODOME:
+        cipher = CD_ChaCha20.new(key=key, nonce=nonce)
+        return cipher.decrypt(data)
+    if HAS_CRYPTOGRAPHY:
+        nonce = bytes(16 - len(nonce)) + nonce  # cryptography wants 16 byte nonces
+        algo = CG_algorithms.ChaCha20(key=key, nonce=nonce)
+        cipher = CG_Cipher(algo, mode=None, backend=CG_default_backend())
+        decryptor = cipher.decryptor()
+        return decryptor.update(data)
+    raise Exception("no chacha20 backend found")
