@@ -1,4 +1,5 @@
 import os
+import threading
 
 from PyQt5.QtCore import Qt, QAbstractListModel, QModelIndex
 from PyQt5.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject
@@ -9,25 +10,29 @@ from electrum.util import WalletFileException, standardize_path
 from electrum.wallet import Abstract_Wallet
 from electrum.plugin import run_hook
 from electrum.lnchannel import ChannelState
+from electrum.daemon import Daemon
 
 from .auth import AuthMixin, auth_protect
 from .qefx import QEFX
 from .qewallet import QEWallet
 from .qewalletdb import QEWalletDB
+from .qewizard import QENewWalletWizard, QEServerConnectWizard
 
 # wallet list model. supports both wallet basenames (wallet file basenames)
 # and whole Wallet instances (loaded wallets)
 class QEWalletListModel(QAbstractListModel):
     _logger = get_logger(__name__)
-    def __init__(self, parent=None):
-        QAbstractListModel.__init__(self, parent)
-        self.wallets = []
 
     # define listmodel rolemap
     _ROLE_NAMES= ('name','path','active')
     _ROLE_KEYS = range(Qt.UserRole, Qt.UserRole + len(_ROLE_NAMES))
     _ROLE_MAP  = dict(zip(_ROLE_KEYS, [bytearray(x.encode()) for x in _ROLE_NAMES]))
 
+    def __init__(self, daemon, parent=None):
+        QAbstractListModel.__init__(self, parent)
+        self.daemon = daemon
+        self.reload()
+
     def rowCount(self, index):
         return len(self.wallets)
 
@@ -35,7 +40,7 @@ class QEWalletListModel(QAbstractListModel):
         return self._ROLE_MAP
 
     def data(self, index, role):
-        (wallet_name, wallet_path, wallet) = self.wallets[index.row()]
+        (wallet_name, wallet_path) = self.wallets[index.row()]
         role_index = role - Qt.UserRole
         role_name = self._ROLE_NAMES[role_index]
         if role_name == 'name':
@@ -43,55 +48,14 @@ class QEWalletListModel(QAbstractListModel):
         if role_name == 'path':
             return wallet_path
         if role_name == 'active':
-            return wallet != None
-
-    def add_wallet(self, wallet_path = None, wallet: Abstract_Wallet = None):
-        if wallet_path == None and wallet == None:
-            return
-        # only add wallet instance if instance not yet in model
-        if wallet:
-            for name,path,w in self.wallets:
-                if w == wallet:
-                    return
-        self.beginInsertRows(QModelIndex(), len(self.wallets), len(self.wallets));
-        if wallet == None:
-            wallet_name = os.path.basename(wallet_path)
-        else:
-            wallet_name = wallet.basename()
-        wallet_path = standardize_path(wallet_path)
-        item = (wallet_name, wallet_path, wallet)
-        self.wallets.append(item);
-        self.endInsertRows();
-
-    def remove_wallet(self, path):
-        i = 0
-        wallets = []
-        remove = -1
-        for wallet_name, wallet_path, wallet in self.wallets:
-            if wallet_path == path:
-                remove = i
-            else:
-                self._logger.debug('HM, %s is not %s', wallet_path, path)
-                wallets.append((wallet_name, wallet_path, wallet))
-            i += 1
-
-        if remove >= 0:
-            self.beginRemoveRows(QModelIndex(), i, i)
-            self.wallets = wallets
-            self.endRemoveRows()
-
-class QEAvailableWalletListModel(QEWalletListModel):
-    def __init__(self, daemon, parent=None):
-        QEWalletListModel.__init__(self, parent)
-        self.daemon = daemon
-        self.reload()
+            return self.daemon.get_wallet(wallet_path) is not None
 
     @pyqtSlot()
     def reload(self):
-        if len(self.wallets) > 0:
-            self.beginRemoveRows(QModelIndex(), 0, len(self.wallets) - 1)
-            self.wallets = []
-            self.endRemoveRows()
+        self._logger.debug('enumerating available wallets')
+        self.beginResetModel()
+        self.wallets = []
+        self.endResetModel()
 
         available = []
         wallet_folder = os.path.dirname(self.daemon.config.get_wallet_path())
@@ -101,61 +65,124 @@ class QEAvailableWalletListModel(QEWalletListModel):
                     available.append(i.path)
         for path in sorted(available):
             wallet = self.daemon.get_wallet(path)
-            self.add_wallet(wallet_path = path, wallet = wallet)
+            self.add_wallet(wallet_path = path)
 
+    def add_wallet(self, wallet_path):
+        self.beginInsertRows(QModelIndex(), len(self.wallets), len(self.wallets))
+        wallet_name = os.path.basename(wallet_path)
+        wallet_path = standardize_path(wallet_path)
+        item = (wallet_name, wallet_path)
+        self.wallets.append(item)
+        self.endInsertRows()
+
+    def remove_wallet(self, path):
+        i = 0
+        wallets = []
+        remove = -1
+        for wallet_name, wallet_path in self.wallets:
+            if wallet_path == path:
+                remove = i
+            else:
+                wallets.append((wallet_name, wallet_path))
+            i += 1
+
+        if remove >= 0:
+            self.beginRemoveRows(QModelIndex(), i, i)
+            self.wallets = wallets
+            self.endRemoveRows()
+
+    @pyqtSlot(str, result=bool)
     def wallet_name_exists(self, name):
-        for wallet_name, wallet_path, wallet in self.wallets:
+        for wallet_name, wallet_path in self.wallets:
             if name == wallet_name:
                 return True
         return False
 
+    @pyqtSlot(str)
+    def updateWallet(self, path):
+        i = 0
+        for wallet_name, wallet_path in self.wallets:
+            if wallet_path == path:
+                mi = self.createIndex(i, i)
+                self.dataChanged.emit(mi, mi, self._ROLE_KEYS)
+                return
+            i += 1
+
 class QEDaemon(AuthMixin, QObject):
-    def __init__(self, daemon, parent=None):
+    _logger = get_logger(__name__)
+
+    _available_wallets = None
+    _current_wallet = None
+    _new_wallet_wizard = None
+    _server_connect_wizard = None
+    _path = None
+    _name = None
+    _use_single_password = False
+    _password = None
+    _loading = False
+
+    _backendWalletLoaded = pyqtSignal([str], arguments=['password'])
+
+    availableWalletsChanged = pyqtSignal()
+    fxChanged = pyqtSignal()
+    newWalletWizardChanged = pyqtSignal()
+    serverConnectWizardChanged = pyqtSignal()
+    loadingChanged = pyqtSignal()
+    requestNewPassword = pyqtSignal()
+
+    walletLoaded = pyqtSignal([str,str], arguments=['name','path'])
+    walletRequiresPassword = pyqtSignal([str,str], arguments=['name','path'])
+    walletOpenError = pyqtSignal([str], arguments=["error"])
+    walletDeleteError = pyqtSignal([str,str], arguments=['code', 'message'])
+
+    def __init__(self, daemon: 'Daemon', parent=None):
         super().__init__(parent)
         self.daemon = daemon
         self.qefx = QEFX(daemon.fx, daemon.config)
+
+        self._backendWalletLoaded.connect(self._on_backend_wallet_loaded)
+
         self._walletdb = QEWalletDB()
         self._walletdb.validPasswordChanged.connect(self.passwordValidityCheck)
-
-    _logger = get_logger(__name__)
-    _loaded_wallets = QEWalletListModel()
-    _available_wallets = None
-    _current_wallet = None
-    _path = None
-    _use_single_password = False
-    _password = None
-
-    walletLoaded = pyqtSignal()
-    walletRequiresPassword = pyqtSignal()
-    activeWalletsChanged = pyqtSignal()
-    availableWalletsChanged = pyqtSignal()
-    walletOpenError = pyqtSignal([str], arguments=["error"])
-    fxChanged = pyqtSignal()
-    walletDeleteError = pyqtSignal([str,str], arguments=['code', 'message'])
+        self._walletdb.walletOpenProblem.connect(self.onWalletOpenProblem)
 
     @pyqtSlot()
     def passwordValidityCheck(self):
         if not self._walletdb._validPassword:
-            self.walletRequiresPassword.emit()
+            self.walletRequiresPassword.emit(self._name, self._path)
+
+    @pyqtSlot(str)
+    def onWalletOpenProblem(self, error):
+        self.walletOpenError.emit(error)
 
     @pyqtSlot()
     @pyqtSlot(str)
     @pyqtSlot(str, str)
-    def load_wallet(self, path=None, password=None):
-        if path == None:
-            self._path = self.daemon.config.get('gui_last_wallet')
+    def loadWallet(self, path=None, password=None):
+        if path is None:
+            self._path = self.daemon.config.get('wallet_path') # command line -w option
+            if self._path is None:
+                self._path = self.daemon.config.get('gui_last_wallet')
         else:
             self._path = path
         if self._path is None:
             return
 
         self._path = standardize_path(self._path)
+        self._name = os.path.basename(self._path)
+
         self._logger.debug('load wallet ' + str(self._path))
 
+        # map empty string password to None
+        if password == '':
+            password = None
+
         if not password:
             password = self._password
 
-        if self._path not in self.daemon._wallets:
+        wallet_already_open = self._path in self.daemon._wallets
+
+        if not wallet_already_open:
             # pre-checks, let walletdb trigger any necessary user interactions
             self._walletdb.path = self._path
             self._walletdb.password = password
@@ -163,38 +190,64 @@ class QEDaemon(AuthMixin, QObject):
             if not self._walletdb.ready:
                 return
 
-        try:
-            wallet = self.daemon.load_wallet(self._path, password)
-            if wallet != None:
-                self._loaded_wallets.add_wallet(wallet_path=self._path, wallet=wallet)
-                self._current_wallet = QEWallet.getInstanceFor(wallet)
-                self._current_wallet.password = password
-                self.walletLoaded.emit()
+        def load_wallet_task():
+            self._loading = True
+            self.loadingChanged.emit()
+
+            try:
+                local_password = password # need this in local scope
+                wallet = self.daemon.load_wallet(self._path, local_password)
+
+                if wallet is None:
+                    self._logger.info('could not open wallet')
+                    self.walletOpenError.emit('could not open wallet')
+                    return
+
+                if wallet_already_open:
+                    # wallet already open. daemon.load_wallet doesn't mind, but
+                    # we need the correct current wallet password below
+                    local_password = QEWallet.getInstanceFor(wallet).password
 
                 if self.daemon.config.get('single_password'):
-                    self._use_single_password = self.daemon.update_password_for_directory(old_password=password, new_password=password)
-                    self._password = password
+                    self._use_single_password = self.daemon.update_password_for_directory(old_password=local_password, new_password=local_password)
+                    self._password = local_password
                     self.singlePasswordChanged.emit()
                     self._logger.info(f'use single password: {self._use_single_password}')
                 else:
                     self._logger.info('use single password disabled by config')
 
                 self.daemon.config.save_last_wallet(wallet)
+
                 run_hook('load_wallet', wallet)
-            else:
-                self._logger.info('could not open wallet')
-                self.walletOpenError.emit('could not open wallet')
-        except WalletFileException as e:
-            self._logger.error(str(e))
-            self.walletOpenError.emit(str(e))
+
+                self._backendWalletLoaded.emit(local_password)
+            except WalletFileException as e:
+                self._logger.error(f"load_wallet_task errored opening wallet: {e!r}")
+                self.walletOpenError.emit(str(e))
+            finally:
+                self._loading = False
+                self.loadingChanged.emit()
+
+        threading.Thread(target=load_wallet_task, daemon=True).start()
+
+    @pyqtSlot()
+    @pyqtSlot(str)
+    def _on_backend_wallet_loaded(self, password = None):
+        self._logger.debug('_on_backend_wallet_loaded')
+        wallet = self.daemon._wallets[self._path]
+        self._current_wallet = QEWallet.getInstanceFor(wallet)
+        self.availableWallets.updateWallet(self._path)
+        self._current_wallet.password = password if password else None
+        self.walletLoaded.emit(self._name, self._path)
+
 
     @pyqtSlot(QEWallet)
     @pyqtSlot(QEWallet, bool)
     @pyqtSlot(QEWallet, bool, bool)
-    def check_then_delete_wallet(self, wallet, confirm_requests=False, confirm_balance=False):
+    def checkThenDeleteWallet(self, wallet, confirm_requests=False, confirm_balance=False):
         if wallet.wallet.lnworker:
             lnchannels = wallet.wallet.lnworker.get_channel_objects()
-            if any([channel.get_state() != ChannelState.REDEEMED for channel in lnchannels.values()]):
+            if any([channel.get_state() != ChannelState.REDEEMED and not channel.is_backup() for channel in lnchannels.values()]):
                 self.walletDeleteError.emit('unclosed_channels', _('There are still channels that are not fully closed'))
                 return
 
@@ -210,38 +263,32 @@ class QEDaemon(AuthMixin, QObject):
 
         self.delete_wallet(wallet)
 
-    @pyqtSlot(QEWallet)
-    @auth_protect
+    @auth_protect(message=_('Really delete this wallet?'))
     def delete_wallet(self, wallet):
         path = standardize_path(wallet.wallet.storage.path)
         self._logger.debug('deleting wallet with path %s' % path)
         self._current_wallet = None
         # TODO walletLoaded signal is confusing
-        self.walletLoaded.emit()
+        self.walletLoaded.emit(None, None)
 
         if not self.daemon.delete_wallet(path):
             self.walletDeleteError.emit('error', _('Problem deleting wallet'))
             return
 
-        self.activeWallets.remove_wallet(path)
         self.availableWallets.remove_wallet(path)
 
-    @pyqtProperty('QString')
-    def path(self):
-        return self._path
+    @pyqtProperty(bool, notify=loadingChanged)
+    def loading(self):
+        return self._loading
 
     @pyqtProperty(QEWallet, notify=walletLoaded)
     def currentWallet(self):
         return self._current_wallet
 
-    @pyqtProperty(QEWalletListModel, notify=activeWalletsChanged)
-    def activeWallets(self):
-        return self._loaded_wallets
-
-    @pyqtProperty(QEAvailableWalletListModel, notify=availableWalletsChanged)
+    @pyqtProperty(QEWalletListModel, notify=availableWalletsChanged)
     def availableWallets(self):
         if not self._available_wallets:
-            self._available_wallets = QEAvailableWalletListModel(self.daemon)
+            self._available_wallets = QEWalletListModel(self.daemon)
 
         return self._available_wallets
 
@@ -260,24 +307,43 @@ class QEDaemon(AuthMixin, QObject):
 
     @pyqtSlot(result=str)
     def suggestWalletName(self):
+        # FIXME why not use util.get_new_wallet_name ?
         i = 1
         while self.availableWallets.wallet_name_exists(f'wallet_{i}'):
             i = i + 1
         return f'wallet_{i}'
 
-    requestNewPassword = pyqtSignal()
     @pyqtSlot()
-    @auth_protect
-    def start_change_password(self):
+    @auth_protect(method='wallet')
+    def startChangePassword(self):
         if self._use_single_password:
             self.requestNewPassword.emit()
         else:
             self.currentWallet.requestNewPassword.emit()
 
-    @pyqtSlot(str)
-    def set_password(self, password):
+    @pyqtSlot(str, result=bool)
+    def setPassword(self, password):
         assert self._use_single_password
-        self._logger.debug('about to set password for ALL wallets')
-        self.daemon.update_password_for_directory(old_password=self._password, new_password=password)
+        assert password
+        if not self.daemon.update_password_for_directory(old_password=self._password, new_password=password):
+            return False
         self._password = password
+        return True
 
+    @pyqtProperty(QENewWalletWizard, notify=newWalletWizardChanged)
+    def newWalletWizard(self):
+        if not self._new_wallet_wizard:
+            self._new_wallet_wizard = QENewWalletWizard(self)
+
+        return self._new_wallet_wizard
+
+    @pyqtProperty(QEServerConnectWizard, notify=serverConnectWizardChanged)
+    def serverConnectWizard(self):
+        if not self._server_connect_wizard:
+            self._server_connect_wizard = QEServerConnectWizard(self)
+
+        return self._server_connect_wizard
+
+    @pyqtSlot()
+    def startNetwork(self):
+        self.daemon.start_network()
