@@ -1,20 +1,20 @@
 from binascii import hexlify, unhexlify
 import traceback
 import sys
-from typing import NamedTuple, Any, Optional, Dict, Union, List, Tuple, TYPE_CHECKING
+from typing import NamedTuple, Any, Optional, Dict, Union, List, Tuple, TYPE_CHECKING, Sequence
 
-from electrum.util import bfh, bh2u, versiontuple, UserCancelled, UserFacingException
+from electrum.util import bfh, versiontuple, UserCancelled, UserFacingException
 from electrum.bip32 import BIP32Node
+from electrum import descriptor
 from electrum import constants
 from electrum.i18n import _
 from electrum.plugin import Device, runs_in_hwd_thread
-from electrum.transaction import Transaction, PartialTransaction, PartialTxInput, PartialTxOutput
+from electrum.transaction import Transaction, PartialTransaction, PartialTxInput, PartialTxOutput, Sighash
 from electrum.keystore import Hardware_KeyStore
 from electrum.base_wizard import ScriptTypeNotSupported
 
 from ..hw_wallet import HW_PluginBase
-from ..hw_wallet.plugin import (is_any_tx_output_on_change_branch, trezor_validate_op_return_output_and_get_data,
-                                get_xpubs_and_der_suffixes_from_txinout)
+from ..hw_wallet.plugin import is_any_tx_output_on_change_branch, trezor_validate_op_return_output_and_get_data
 
 if TYPE_CHECKING:
     from .client import SafeTClient
@@ -29,9 +29,6 @@ class SafeTKeyStore(Hardware_KeyStore):
 
     plugin: 'SafeTPlugin'
 
-    def get_client(self, force_pair=True):
-        return self.plugin.get_client(self, force_pair)
-
     def decrypt_message(self, sequence, message, password):
         raise UserFacingException(_('Encryption and decryption are not implemented by {}').format(self.device))
 
@@ -180,7 +177,7 @@ class SafeTPlugin(HW_PluginBase):
             import threading
             settings = self.request_safe_t_init_settings(wizard, method, self.device)
             t = threading.Thread(target=self._initialize_device_safe, args=(settings, method, device_id, wizard, handler))
-            t.setDaemon(True)
+            t.daemon = True
             t.start()
             exit_code = wizard.loop.exec_()
             if exit_code != 0:
@@ -244,7 +241,7 @@ class SafeTPlugin(HW_PluginBase):
             client.load_device_by_xprv(item, pin, passphrase_protection,
                                        label, language)
 
-    def _make_node_path(self, xpub, address_n):
+    def _make_node_path(self, xpub: str, address_n: Sequence[int]):
         bip32node = BIP32Node.from_xkey(xpub)
         node = self.types.HDNodeType(
             depth=bip32node.depth,
@@ -303,7 +300,8 @@ class SafeTPlugin(HW_PluginBase):
         outputs = self.tx_outputs(tx, keystore=keystore)
         signatures = client.sign_tx(self.get_coin_name(), inputs, outputs,
                                     lock_time=tx.locktime, version=tx.version)[0]
-        signatures = [(bh2u(x) + '01') for x in signatures]
+        sighash = Sighash.to_sigbytes(Sighash.ALL).hex()
+        signatures = [(x.hex() + sighash) for x in signatures]
         tx.update_signatures(signatures)
 
     @runs_in_hwd_thread
@@ -323,14 +321,9 @@ class SafeTPlugin(HW_PluginBase):
         script_type = self.get_safet_input_script_type(wallet.txin_type)
 
         # prepare multisig, if available:
-        xpubs = wallet.get_master_public_keys()
-        if len(xpubs) > 1:
-            pubkeys = wallet.get_public_keys(address)
-            # sort xpubs using the order of pubkeys
-            sorted_pairs = sorted(zip(pubkeys, xpubs))
-            multisig = self._make_multisig(
-                wallet.m,
-                [(xpub, deriv_suffix) for pubkey, xpub in sorted_pairs])
+        desc = wallet.get_script_descriptor_for_address(address)
+        if multi := desc.get_simple_multisig():
+            multisig = self._make_multisig(multi)
         else:
             multisig = None
 
@@ -348,12 +341,13 @@ class SafeTPlugin(HW_PluginBase):
                     assert isinstance(tx, PartialTransaction)
                     assert isinstance(txin, PartialTxInput)
                     assert keystore
-                    if len(txin.pubkeys) > 1:
-                        xpubs_and_deriv_suffixes = get_xpubs_and_der_suffixes_from_txinout(tx, txin)
-                        multisig = self._make_multisig(txin.num_sig, xpubs_and_deriv_suffixes)
+                    desc = txin.script_descriptor
+                    assert desc
+                    if multi := desc.get_simple_multisig():
+                        multisig = self._make_multisig(multi)
                     else:
                         multisig = None
-                    script_type = self.get_safet_input_script_type(txin.script_type)
+                    script_type = self.get_safet_input_script_type(desc.to_legacy_electrum_script_type())
                     txinputtype = self.types.TxInputType(
                         script_type=script_type,
                         multisig=multisig)
@@ -378,22 +372,27 @@ class SafeTPlugin(HW_PluginBase):
 
         return inputs
 
-    def _make_multisig(self, m, xpubs):
-        if len(xpubs) == 1:
-            return None
-        pubkeys = [self._make_node_path(xpub, deriv) for xpub, deriv in xpubs]
+    def _make_multisig(self, desc: descriptor.MultisigDescriptor):
+        pubkeys = []
+        for pubkey_provider in desc.pubkeys:
+            assert not pubkey_provider.is_range()
+            assert pubkey_provider.extkey is not None
+            xpub = pubkey_provider.pubkey
+            der_suffix = pubkey_provider.get_der_suffix_int_list()
+            pubkeys.append(self._make_node_path(xpub, der_suffix))
         return self.types.MultisigRedeemScriptType(
             pubkeys=pubkeys,
             signatures=[b''] * len(pubkeys),
-            m=m)
+            m=desc.thresh)
 
     def tx_outputs(self, tx: PartialTransaction, *, keystore: 'SafeTKeyStore'):
 
         def create_output_by_derivation():
-            script_type = self.get_safet_output_script_type(txout.script_type)
-            if len(txout.pubkeys) > 1:
-                xpubs_and_deriv_suffixes = get_xpubs_and_der_suffixes_from_txinout(tx, txout)
-                multisig = self._make_multisig(txout.num_sig, xpubs_and_deriv_suffixes)
+            desc = txout.script_descriptor
+            assert desc
+            script_type = self.get_safet_output_script_type(desc.to_legacy_electrum_script_type())
+            if multi := desc.get_simple_multisig():
+                multisig = self._make_multisig(multi)
             else:
                 multisig = None
             my_pubkey, full_path = keystore.find_my_pubkey_in_txinout(txout)
